#ifndef AVI_H
#define AVI_H

#include <iostream>

//-----------------------------------------------------------------------------
//
//	RIFF header
//
//-----------------------------------------------------------------------------
struct RIFFHeader
{
	RIFFHeader( unsigned long _fileSize );

	char			fourCC[ 4 ];
	unsigned long	fileSize;
	char			fileType[ 4 ];
};

std::ostream& operator<<( std::ostream& _out, const RIFFHeader& _h );

//-----------------------------------------------------------------------------
//
//	RIFF chunk header
//
//-----------------------------------------------------------------------------
struct ChunkHeader
{
	ChunkHeader( const std::string& _fourCC, unsigned long _chunkSize );

	char			fourCC[ 4 ];
	unsigned long	chunkSize;
};

std::ostream& operator<<( std::ostream& _out, const ChunkHeader& _h );

//-----------------------------------------------------------------------------
//
//	RIFF list header
//
//-----------------------------------------------------------------------------
struct ListHeader
{
	ListHeader( const std::string& _listType, unsigned long _listSize );

	char			fourCC[ 4 ];
	unsigned long	listSize;
	char			listType[ 4 ];
};

std::ostream& operator<<( std::ostream& _out, const ListHeader& _h );

//-----------------------------------------------------------------------------
//
//	AVI main header
//
//-----------------------------------------------------------------------------
struct AVIMainHeader
{
	AVIMainHeader( unsigned long _width, unsigned long _height, int _framesPerSecond, unsigned long _frameCount );

    char			fourCC[ 4 ];
    unsigned long	headerSize;
    unsigned long	microSecondsPerFrame;
    unsigned long	maxBytesPerSecond;
    unsigned long	paddingGranularity;
    unsigned long	flags;
    unsigned long	totalFrames;
    unsigned long	initialFrames;
    unsigned long	streams;
    unsigned long	suggestedBufferSize;
    unsigned long	width;
    unsigned long	height;
    unsigned long	reserved[ 4 ];
};

std::ostream& operator<<( std::ostream& _out, const AVIMainHeader& _h );

//-----------------------------------------------------------------------------
//
//	AVI stream header
//
//-----------------------------------------------------------------------------
struct AVIStreamHeader
{
	AVIStreamHeader( int _width, int _height, int _framesPerSecond, int _frameCount );

	char			fourCC[ 4 ];
	unsigned long	headerSize;
	char			streamType[ 4 ];
	char			streamHandler[ 4 ];
	unsigned long	flags;
	unsigned short	priority;
	unsigned short	language;
	unsigned long	initialFrames;
	unsigned long	scale;
	unsigned long	rate;
	unsigned long	start;
	unsigned long	length;
	unsigned long	suggestedBufferSize;
	unsigned long	quality;
	unsigned long	sampleSize;

	struct
	{
	 short int left;
	 short int top;
	 short int right;
	 short int bottom;
	}  frame;
};

std::ostream& operator<<( std::ostream& _out, const AVIStreamHeader& _h );

struct AVIStreamFormatHeader
{
	AVIStreamFormatHeader( int _width, int _height );

	unsigned long	headerSize;
	long			width;
	long			height;
	short			colorPlanes;
	short			bitsPerPixel;
	unsigned long	compression;
	unsigned long	imageSize;
	long			horizontalRes;
	long			verticalRes;
	unsigned long	colorCount;
	unsigned long	importantColors;
};

std::ostream& operator<<( std::ostream& _out, const AVIStreamFormatHeader& _h );

struct AVIIndexEntry
{
	AVIIndexEntry( const std::string& _chunkID, int _chunkOffset, int _chunkSize );

	char			chunkID[ 4 ];
	unsigned long	flags;
	unsigned long	offset;
	unsigned long	size;
};

std::ostream& operator<<( std::ostream& _out, const AVIIndexEntry& _e );

#endif
