#include "BMP.h"

BMPFileHeader::BMPFileHeader( long _width, long _height )
{
	int padSize = 4 - ( ( _width * 3 ) % 4 );
	if( padSize == 4 ) padSize = 0;

	magic		= 0x4D42;
	fileSize	= 54 + ( ( ( _width * 3 ) + padSize ) * _height );
	reserved_1	= 0;
	reserved_2	= 0;
	dataOffset	= 54;
}

std::ostream& operator<<( std::ostream& _out, const BMPFileHeader& _h )
{
	_out.write( reinterpret_cast< const char* >( &_h.magic ), sizeof( _h.magic ) );
	_out.write( reinterpret_cast< const char* >( &_h.fileSize ), sizeof( _h.fileSize ) );
	_out.write( reinterpret_cast< const char* >( &_h.reserved_1 ), sizeof( _h.reserved_1 ) );
	_out.write( reinterpret_cast< const char* >( &_h.reserved_2 ), sizeof( _h.reserved_2 ) );
	_out.write( reinterpret_cast< const char* >( &_h.dataOffset ), sizeof( _h.dataOffset ) );
	return _out;
}

BMPInfoHeader::BMPInfoHeader( long _width, long _height )
{
	int padSize = 4 - ( ( _width * 3 ) % 4 );
	if( padSize == 4 ) padSize = 0;

	headerSize		= 40;
	width			= _width;
	height			= _height;
	colorPlanes		= 1;
	bitsPerPixel	= 24;
	compression		= 0;	// BI_RGB
	imageSize		= ( ( _width * 3 ) + padSize ) * height;
	horizontalRes	= 0;
	verticalRes		= 0;
	colorCount		= 0;
	importantColors	= 0;
}

std::ostream& operator<<( std::ostream& _out, const BMPInfoHeader& _h )
{
	_out.write( reinterpret_cast< const char* >( &_h.headerSize ), sizeof( _h.headerSize ) );
	_out.write( reinterpret_cast< const char* >( &_h.width ), sizeof( _h.width ) );
	_out.write( reinterpret_cast< const char* >( &_h.height ), sizeof( _h.height ) );
	_out.write( reinterpret_cast< const char* >( &_h.colorPlanes ), sizeof( _h.colorPlanes ) );
	_out.write( reinterpret_cast< const char* >( &_h.bitsPerPixel ), sizeof( _h.bitsPerPixel ) );
	_out.write( reinterpret_cast< const char* >( &_h.compression ), sizeof( _h.compression ) );
	_out.write( reinterpret_cast< const char* >( &_h.imageSize ), sizeof( _h.imageSize ) );
	_out.write( reinterpret_cast< const char* >( &_h.horizontalRes ), sizeof( _h.horizontalRes ) );
	_out.write( reinterpret_cast< const char* >( &_h.verticalRes ), sizeof( _h.verticalRes ) );
	_out.write( reinterpret_cast< const char* >( &_h.colorCount ), sizeof( _h.colorCount ) );
	_out.write( reinterpret_cast< const char* >( &_h.importantColors ), sizeof( _h.importantColors ) );
	return _out;
}
