#include <exception>
#include <fstream>
#include <sstream>
#include "Configuration.h"

Configuration::Configuration()
{
	SetDefault();
}

Configuration::Configuration( const std::string& _fileName )
{
	std::ifstream file( _fileName.c_str() );
	if( !file )
	{
		std::ostringstream ss;
		ss << "config: cannot load configuration file " << _fileName;
		// throw std::runtime_error( ss.str().c_str() );
		return;
	}

	std::string line;
	std::string token;
	while( std::getline( file, line ) )
	{
		if( line.empty() || line[ 0 ] == ';' )
		{
			continue;
		}

		std::istringstream ss( line );
		ss >> token;
		if( token == "Nodes" )
		{
			ss >> nodeCount;
		}
		else if( token == "NodeRadius" )
		{
			ss >> nodeRadius;
		}
		else if( token == "KnowingTime" )
		{
			ss >> knowingTime;
		}
		else if( token == "SimulationSeconds" )
		{
			ss >> simulationTime;
		}
		else if( token == "CellDistanceWeight" )
		{
			ss >> cellDistanceWeight;
		}
		else if( token == "NodeSpeedMultiplier" )
		{
			ss >> nodeSpeedMultiplier;
		}
		else if( token == "WaitingTimeExponent" )
		{
			ss >> waitingTimeExponent;
		}
		else if( token == "WaitingTimeUpperBound" )
		{
			ss >> waitingTimeUpperBound;
		}
		else if( token == "BucketsPerSide" )
		{
			ss >> bucketCountPerSide;
		}
		else if( token == "ShowOnlyMeetAndLeave" )
		{
			ss >> showOnlyMeetAndLeave;
		}
		else if( token == "SaveVideo" )
		{
			ss >> saveVideo;
		}
		else if( token == "SaveImages" )
		{
			ss >> saveImages;
		}
		else if( token == "VideoFPS" )
		{
			ss >> videoFPS;
			if( videoFPS <= 0 )
			{
				// throw std::runtime_error( "config: VideoFPS must be > 0" );
			}
		}
		else if( token == "ImageTimeInterval" )
		{
			ss >> imageTimeInterval;
		}
		else if( token == "VideoFileName" )
		{
			ss >> videoFileName;
		}
		else if( token == "VideoWidth" )
		{
			ss >> videoWidth;
		}
		else if( token == "VideoHeight" )
		{
			ss >> videoHeight;
		}
		else if( token == "DrawNodes" )
		{
			ss >> drawNodes;
		}
		else if( token == "DrawNodeRadius" )
		{
			ss >> drawNodeRadius;
		}
		else if( token == "DrawCellWeights" )
		{
			ss >> drawCellWeights;
		}
		else if( token == "DrawBuckets" )
		{
			ss >> drawBuckets;
		}
		else if( token == "TargetNodeID" )
		{
			ss >> targetNodeID;
		}
		else if( token == "DrawGrid" )
		{
			ss >> drawGrid;
		}
		else if( token == "ShowTrace" )
		{
			ss >> showTrace;
		}
		else if( token == "SaveTraceFile" )
		{
			ss >> saveTraceFile;
		}
		else if( token == "TraceFileName" )
		{
			ss >> traceFileName;
		}
		else if( token == "ShowProgress" )
		{
			ss >> showProgress;
		}
		else if( token == "ProgressStep" )
		{
			ss >> progressStep;
		}
	}

	file.close();
}

void Configuration::SetDefault()
{
	nodeCount = 3;
	nodeRadius = 0.2;
	knowingTime = 1728000;
	simulationTime = 10.0;
	cellDistanceWeight = 0.5;
	nodeSpeedMultiplier = 1.0;
	waitingTimeExponent = 1.45;
	waitingTimeUpperBound = 240.0;

	bucketCountPerSide = 0;

	showOnlyMeetAndLeave = false;
	saveVideo = false;
	saveImages = false;
	videoFPS = 0;
	imageTimeInterval = 0;
	videoWidth = 200;
	videoHeight = 200;
	drawNodes = true;
	drawNodeRadius = true;
	drawCellWeights = false;
	drawBuckets = false;
	targetNodeID = 0;
	drawGrid = false;

	saveTraceFile = false;

	showTrace = true;
	showProgress = false;
	progressStep = 0;
}

int Configuration::GetNodeCount() const
{
	return nodeCount;
}

Real Configuration::GetNodeRadius() const
{
	return nodeRadius;
}

Real Configuration::GetKnowingTime() const
{
	return knowingTime;
}

Real Configuration::GetSimulationTime() const
{
	return simulationTime;
}

Real Configuration::GetCellDistanceWeight() const
{
	return cellDistanceWeight;
}

Real Configuration::GetNodeSpeedMultiplier() const
{
	return nodeSpeedMultiplier;
}

Real Configuration::GetWaitingTimeExponent() const
{
	return waitingTimeExponent;
}

Real Configuration::GetWaitingTimeUpperBound() const
{
	return 	waitingTimeUpperBound;
}

int Configuration::GetBucketCountPerSide() const
{
	return bucketCountPerSide;
}

bool Configuration::IsShowOnlyMeetAndLeaveEnabled() const
{
	return showOnlyMeetAndLeave;
}

bool Configuration::IsSaveVideoEnabled() const
{
	return saveVideo;
}

bool Configuration::IsSaveImagesEnabled() const
{
	return saveImages;
}

int Configuration::GetVideoFPS() const
{
	return videoFPS;
}

Real Configuration::GetImageTimeInterval() const
{
	return imageTimeInterval;
}

std::string Configuration::GetVideoFileName() const
{
	return videoFileName;
}

int Configuration::GetVideoWidth() const
{
	return videoWidth;
}

int Configuration::GetVideoHeight() const
{
	return videoHeight;
}

bool Configuration::IsDrawNodesEnabled() const
{
	return drawNodes;
}

bool Configuration::IsDrawNodeRadiusEnabled() const
{
	return drawNodeRadius;
}

bool Configuration::IsDrawCellWeightsEnabled() const
{
	return drawCellWeights;
}

bool Configuration::IsDrawBucketsEnabled() const
{
	return drawBuckets;
}

int Configuration::GetTargetNodeID() const
{
	return targetNodeID;
}

bool Configuration::IsDrawGridEnabled() const
{
	return drawGrid;
}

bool Configuration::IsShowTraceEnabled() const
{
	return showTrace;
}

bool Configuration::IsSaveTraceFileEnabled() const
{
	return saveTraceFile;
}

std::string Configuration::GetTraceFileName() const
{
	return traceFileName;
}

bool Configuration::IsShowProgressEnabled() const
{
	return showProgress;
}

int Configuration::GetProgressStep() const
{
	return progressStep;
}
