#include "Event.h"

Event::Event() : type( Event::INVALID ), firstNode( NULL ), secondNode( NULL ), time( 0 )
{
}

Event::Event( Type _type, Node* _n1, Node* _n2, Real _time )
	: type( _type ), firstNode( _n1 ), secondNode( _n2 ), time( _time )
{
}

bool Event::operator<( const Event& _other ) const
{
	// the event with greater time has lower priority
	return( time > _other.time );
}


std::ostream& operator<<( std::ostream& _out, const Event& _e )
{	
	_out.width( 6 );
	_out.setf( std::ios::fixed, std::ios::floatfield );
	_out.precision( 3 );
	_out << _e.time;

	_out.width( 6 );
	switch( _e.type )
	{
	case Event::START_MOVING:	_out << "START_MOVING";		break;
	case Event::START_WAITING:	_out << "START_WAITING";	break;
	case Event::END_MOVING:		_out << "END_MOVING";		break;
	case Event::END_WAITING:	_out << "END_WAITING";		break;

	case Event::MEET:			_out << "M";		break;
	case Event::LEAVE:			_out << "L";		break;
	case Event::IMAGE:			_out << "IMAGE";	break;

		
	default:
//	case Event::INVALID:		_out << "INVALID";		
	break;
	}

	if( _e.firstNode )
	{
		_out.width( 6 );
		_out << _e.firstNode->GetID();
	}
	if( _e.secondNode )
	{
		_out.width( 6 );
		_out << _e.secondNode->GetID();
	}

	_out << std::endl;

	return _out;
}


Real Event::getTime()
{
	return time;
}


