#ifndef NETWORK_H
#define NETWORK_H

#include <vector>
#include <set>
#include "Configuration.h"
#include "Vector2.h"
#include "NodeCheckList.h"

class Node;

enum BucketProcessMode
{
	BUCKET_UPDATE,
	BUCKET_CREATE_NODE_CHECK_LIST,
};

class Network
{
public:

	Network( Configuration& _config );
	~Network();

	size_t		AddNode( Real _x, Real _y, int node_count );

	void		InitNodes();

	size_t		GetNodeCount() const;
	int		GetNumberOfNodes();
	Node*		GetNode( size_t _index );
	
	Real		GetNodeSpeedMultiplier()	const;
	Real		GetWaitingTimeExponent()	const;
	Real		GetWaitingTimeUpperBound()	const;
	Real		GetCellDistanceWeight()		const;

	// cells

	int			GetCellCount()					const;
	int			GetCellCountPerSide()				const;
	Real			GetCellLength()					const;
	int			GetCellIndexFromPos( const Vector2& _pos ) 	const;
	Vector2			GetCellPos( int _cellIndex ) 			const;
	Vector2			GetRandomPointInCell( int _cellIndex ) 		const;

	// buckets

	int			GetBucketCount()		const;
	int			GetBucketCountPerSide()		const;
	Real			GetBucketLength()		const;
	Vector2			GetBucketPos( int _bucketIndex );
	void			ProcessBuckets( Node* _node, BucketProcessMode _mode );
	
	NodeCheckList*		GetNodesToCheck( Node* _node );

	bool			CheckPassage( int _bucketIndex, Node* _node );

	std::set< Node* >& 	TEMP_GetBucket( int i );
	
	
	void		RaiseCellWeight( int index );
	int		GetCellWeight( int index );
	void		PrintCellVisits();
	

private:

	Configuration&	config;

	Real	nodeSpeedMultiplier;
	Real	waitingTimeExponent;
	Real	waitingTimeUpperBound;
	Real	cellDistanceWeight;

	std::vector< Node* >	nodes;

	Real			cellLength;
	int			cellCountPerSide;
	int			cellCount;

	std::vector< std::set< Node* > > buckets;

	Real		bucketLength;
	Real		bucketRadius;
	int		bucketCountPerSide;
	int		bucketCount;
	
	
	int*		total_cell_weights;

	NodeCheckList		nodeCheckList;

};

#endif
