#ifndef NODE_H
#define NODE_H

#include <cstddef>
#include <valarray>
#include "Real.h"
#include "Vector2.h"
#include "Network.h"

#include <set>

class Node
{
public:

	enum State
	{
		NEW,		// The node has just been created
		MOVING,		// The node is moving from start to dest
		WAITING,	// The node is waiting in start
	};

	Node( size_t _id, Vector2 _homePos, Network& _network, int node_count, Real _nodeRadius  );

	void		UpdatePosition( Vector2 _pos, Real _time );

	void		WaitRandomTime();
	void		MoveToRandomDestination();

	void		Meet( Node* _other, Real currentTime, Real maxTime );
	void		Leave( Node* _other );

	size_t		GetID()					const;
	State		GetState()				const;

	Vector2		GetPosition()				const;
	Vector2		GetDestination()			const;
	Vector2		ComputePositionAtTime( Real _absTime ) 	const;
	
	Vector2		GetHomePosition()			const;
	int		GetNeighbourContactTime(int index);
	int		GetDestinationCell();
	void		PrintCellWeights();
	
	Real		GetPositionAbsoluteTime()		const;
	Real		GetTravelTime()				const;

	void		InitCellWeights();
	void		SetCellWeight( int _cellIndex, int _seen );
	Real		GetCellWeight( int _cellIndex );

	bool		operator<( const Node& _other );

private:

	Real		ComputeRandomWaitingTime() const;

	Real		DistanceFunction( int _cellIndex ) const;
	Real		SeenFunction( int _seen ) const;

	int		ChooseDestinationCell() const;

	void		NormalizeCellWeights();

private:

	size_t			id;

	State			state;

	Vector2			pos;
	Real			posTime;
	Vector2			dest;

	Real			speed;
	Real			waitTime;

	Vector2			home;

	Real			cellDistanceWeight;
	std::valarray< Real >	cellWeights;
	
	int			currentCell;
	int			destinationCell;
	int			contactCount;
	Real			nodeRadius;
	int			numberOfNodes;
	Real 			densita;

	std::valarray<int>	neighbours_list;			// Id-s of neighbors node
	std::valarray<int>	number_of_visits;			// Total number of visits in a cell
	std::valarray<int>	number_of_nodes_seen;			// Total number of nodes seen in a cell
	std::valarray<int>	number_of_nodes_seen_last_visit;	// Number of nodes seen the last visit in the cell

	Network&		network;
};

#endif
