#include <ctime>
#include <cstdlib>
#include "Random.h"
#include "MersenneTwister.h"


#define USE_MERSENNE_TWISTER

namespace random_generator
{
	unsigned long seed = 0;
	MTRand*	mt = NULL;

	void Init()
	{
		seed = ( unsigned long )time( NULL );

#ifdef USE_MERSENNE_TWISTER

		mt = new MTRand( seed );
		
#else

		srand( seed );

#endif
	}

	void Term()
	{
#ifdef USE_MERSENNE_TWISTER

		delete mt;
		mt = NULL;

#endif
	}

	unsigned long GetSeed()
	{
		return seed;
	}

	Real GetReal()
	{
#ifdef USE_MERSENNE_TWISTER

		return mt->rand();

#else

		return( rand() / ( Real )RAND_MAX );

#endif
	}

	Real GetReal( Real min, Real max )
	{
#ifdef USE_MERSENNE_TWISTER

		Real range = max - min;
		Real number = mt->rand( range );
		return number + min;

#else

		Real range = max - min;
		Real number = ( rand() / ( Real )RAND_MAX );
		return( ( number * range ) + min );

#endif

	}
	
}
