#ifndef SIMULATION_H
#define SIMULATION_H

#include <queue>
#include <vector>
#include <fstream>
#include "Real.h"
#include "Network.h"
#include "Event.h"
#include "Color.h"
#include "Video.h"
#include "Configuration.h"

class Simulation
{
public:

	Simulation( Network& _n, const Configuration& _config );
	~Simulation();

	void			Start();
	std::ofstream	tracefile;
	std::ofstream	logfile;
	void	PrintEvent( Event e, const char* eventType );	
	void	PrintEvent( Event e );	
	
private:

	void	CheckContacts( Node* node );
	bool	CheckContactWithNode( Node* node, Node* otherNode );
	Real	min( Real a, Real b );
	bool**	meetInPlace;
	int		nodeNumber;

	void	SaveImage();
	
	typedef std::priority_queue< Event, std::vector< Event >, std::less< Event > > QueueType;


	const Configuration&	config;

//	std::ofstream*		traceFile;

	Network&		network;
	QueueType		eventQueue;
	Real			currentTime;

	// simulation parameters
	
	Real			knowingTime;
	Real			maxTime;
	Real			meetRadius;

	// image and video

	int						nextImageIndex;
	std::vector< Color >	nodeColors;
	Video*					video;

	// constants

	enum
	{
		NODE_PIXEL_RADIUS	= 2,
	};
	
};

#endif
