#ifndef VECTOR2_H
#define VECTOR2_H

#include <cmath>
#include "Real.h"
#include "Random.h"

class Vector2
{
public:

	Vector2() : x( 0 ), y( 0 )
	{
	}

	Vector2( Real _x, Real _y ) : x( _x ), y( _y )
	{
	}

	Real GetLength() const
	{
		return( static_cast< Real >( sqrt( static_cast< double >( x * x + y * y ) ) ) );
	}

	void Normalize()
	{
		Real length = GetLength();
		if( length != 0 )
		{
			x /= length;
			y /= length;
		}
	}

	Vector2 operator+( const Vector2& _rhs ) const
	{
		return Vector2( x + _rhs.x, y + _rhs.y );
	}

	Vector2 operator-( const Vector2& _rhs ) const
	{
		return Vector2( x - _rhs.x, y - _rhs.y );
	}

	Vector2 operator*( Real _r ) const
	{
		return Vector2( x * _r, y * _r );
	}

	Real Dot( const Vector2& _rhs ) const
	{
		return( ( x * _rhs.x ) + ( y * _rhs.y ) );
	}

	Vector2& operator+=( const Vector2& _rhs )
	{
		x += _rhs.x;
		y += _rhs.y;
		return *this;
	}

	Vector2& operator-=( const Vector2& _rhs )
	{
		x -= _rhs.x;
		y -= _rhs.y;
		return *this;
	}

	Vector2& operator*=( Real _r )
	{
		x *= _r;
		y *= _r;
		return *this;
	}
	
	Real GetX()
	{
		return x;
	}
	
	Real GetY()
	{
		return y;
	}

	Real x;
	Real y;
};

#endif
