/** Added from Koli **/

#define NDEBUG

/*********************/

#include <cassert>
// #include <windows.h>
#include <fstream>
#include "Video.h"
#include "AVI.h"

Video::Video( int _width, int _height, int _framesPerSecond )
{
	width = _width;
	height = _height;
	framesPerSecond = _framesPerSecond;
}

Video::~Video()
{
	for( size_t i = 0; i < frames.size(); ++i )
	{
		delete frames[ i ];
	}
	frames.clear();
}

size_t Video::GetFrameCount() const
{
	return frames.size();
}

Image* Video::GetFrame( size_t _index )
{
	assert( _index < frames.size() );
	return frames[ _index ];
}

Image* Video::CreateFrame( Color _fillColor )
{
	Image* newFrame = new Image( width, height, _fillColor );
	frames.push_back( newFrame );
	return newFrame;
}

bool Video::AddFrame( Image* _frame )
{
	if( _frame->GetWidth() != width || _frame->GetHeight() != height )
	{
		return false;
	}

	Image* newFrame = new Image( *_frame );
	frames.push_back( newFrame );

	return true;
}

bool Video::SaveAVI( const std::string& _fileName ) const
{
	std::ofstream file( _fileName.c_str(), std::ios::binary );
	if( !file )
	{
		return false;
	}

	int mainHeaderSize = 64;

	int padSize = 4 - ( ( width * 3 ) % 4 );
	if( padSize == 4 ) padSize = 0;

	int frameDataSize = ( ( width * 3 ) + padSize ) * height;

	int streamListSize = 116;
	int firstListSize = 4 + mainHeaderSize + 8 + streamListSize;
	int secondListSize = 4 + ( ( frameDataSize + 8 ) * ( int )frames.size() );
	int indexSize = ( int )frames.size() * 16;

	int fileSize = 4 + 8 + firstListSize + 8 + secondListSize + 8 + indexSize;

	// RIFF header

	RIFFHeader riffHeader( fileSize );
	file << riffHeader;

	// first list

	file << ListHeader( "hdrl", firstListSize );
	AVIMainHeader aviHeader( width, height, framesPerSecond, ( int )frames.size() );
	file << aviHeader;

	file << ListHeader( "strl", streamListSize );

	// strh chunk
	AVIStreamHeader streamHeader( width, height, framesPerSecond, ( int )frames.size() );
	file << streamHeader;

	// strf chunk
	file << ChunkHeader( "strf", 40 );
	AVIStreamFormatHeader streamFormat( width, height );
	file << streamFormat;

	// second list (data)

	ListHeader secondListHeader( "movi", secondListSize );
	file << secondListHeader;

	unsigned char* frameDataBuffer = new unsigned char[ frameDataSize ];
	for( int i = 0; i < frameDataSize; ++i )
	{
		frameDataBuffer[ i ] = 0;
	}

	// write the frames

	for( size_t frameIndex = 0; frameIndex < frames.size(); ++frameIndex )
	{
		ChunkHeader chunkHeader( "00db", frameDataSize );
		file << chunkHeader;

		Image* frame = frames[ frameIndex ];

		int dataBufferOffset = 0;
		int imageOffset = 0;

		for( int row = height - 1; row >= 0; --row )
		{
			for( int col = 0; col < width; ++col )
			{
				int imageOffset = ( row * width ) + col;
				frameDataBuffer[ dataBufferOffset ] = frame->bData[ imageOffset ];
				frameDataBuffer[ dataBufferOffset + 1 ] = frame->gData[ imageOffset ];
				frameDataBuffer[ dataBufferOffset + 2 ] = frame->rData[ imageOffset ];

				++imageOffset;
				dataBufferOffset += 3;
			}
			dataBufferOffset += padSize;
		}

		file.write( reinterpret_cast< const char* >( frameDataBuffer ), frameDataSize );
	}

	delete[] frameDataBuffer;

	// index

	file << ChunkHeader( "idx1", indexSize );
	for( size_t i = 0; i < frames.size(); ++i )
	{
		file << AVIIndexEntry( "00db", 4 + ( int )i * ( frameDataSize + 8 ), frameDataSize );
	}

	file.close();
	return true;
}
